﻿using System;
using System.Collections.Generic;
using SixLabors.ImageSharp;
using SixLabors.ImageSharp.PixelFormats;
using System.Threading.Tasks;
using System.Reflection;
using System.IO;
using Microsoft.AspNetCore.Mvc;
using Timelapse.Data;
namespace Timelapse.Api
{
    [AttributeUsage(AttributeTargets.Class)]
   // [Extension("Tesses","oiajifajdifajedifaji",ReleaseDate = "12/27/2021",Version ="1.0.0.0")]
    public class ExtensionAttribute : Attribute
    {
        public ExtensionAttribute(string name,string id)
        {
            Name=name;
            Id=id;
           ReleaseDate="8/20/1992";
           Version="1.0.0.0";
           
        }
        ///<summary>
        /// name
        ///</summary>
        public string Name {get;set;}
        ///<summary>
        /// something unique
        ///</summary>
        public string Id {get;set;}
        ///<summary>
        /// ex 8/20/1992
        ///</summary>
        public string ReleaseDate {get;set;}
        ///<summary>
        /// ex 1.0.0.0
        ///</summary>
        public string Version {get;set;}
    }
    public class Extensions
    {
        public static IReadOnlyDictionary<string,Type> Components {get {return _components;}}
       internal static Dictionary<string,Type> _components = new Dictionary<string, Type>();
        public static string GetId(TimelapseExtension e)

        {
            return e.__id;
        }
        /// <summary>
        /// Callbacks back to TimelapseUI
        /// </summary>
        /// <value></value>
        public static Callbacks Callbacks {get;set;}
        /// <summary>
        /// UserData
        /// </summary>
        /// <returns>TimelapseNow Folder for user</returns>
        public static string UserData =Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),"TimelapseNow");
       /// <summary>
       /// UserExtensions
       /// </summary>
       /// <returns>Location for Extensions</returns>
        public static string UserExtensions = Path.Combine(UserData,"Extensions");
        /// <summary>
        /// Global Data for app
        /// </summary>
        /// <returns>Timelapse Folder for all users</returns>
        public static string GlobalData = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData),"TimelapseNow");
        /// <summary>
        /// GlobalExtensions
        /// </summary>
        /// <returns>Location for extensions for all users</returns>
        public static string GlobalExtensions = Path.Combine(GlobalData,"Extensions");
        /// <summary>
        /// Dont call this
        /// </summary>
        public static void EnumerateExtensions()
        {
            TimelapseExtensions = new List<TimelapseExtension>();
            Assemblies = new Dictionary<Assembly, string>();
          Attributes  = new Dictionary<ExtensionAttribute, TimelapseExtension>();
            if(Directory.Exists(GlobalExtensions))
            {
                foreach(var item in Directory.GetDirectories(GlobalExtensions))
                {
                    string name = Path.GetFileName(item);
                    string dll = Path.Combine(item,name + ".dll");
                    if(File.Exists(dll))
                    {
                        LoadDll(dll);
                    }
                }
            }
            Directory.CreateDirectory(UserExtensions);
            foreach(var item in Directory.GetDirectories(UserExtensions))
            {
                string name = Path.GetFileName(item);
                string dll = Path.Combine(item,name + ".dll");
                if(File.Exists(dll))
                {
                    LoadDll(dll);
                }
            }
        }
        /// <summary>
        /// Only call this to load extension manually
        /// </summary>
        /// <param name="dll">path to dll</param>
        public static void LoadDll(string dll)
        {
            Assembly asm=Assembly.LoadFrom(dll);
            foreach(var type in asm.GetTypes())
            {
                ExtensionAttribute attr=type.GetCustomAttribute<ExtensionAttribute>();
                if(typeof(TimelapseExtension).IsAssignableFrom(type) && attr != null)
                {
                    Console.WriteLine("Before Create Instance");
                    //load assembly info
                    
                    var item=(TimelapseExtension)Activator.CreateInstance(type);
                    Console.WriteLine("Affter Create Instance");
                    item.__id=attr.Id;
                    item.Name = attr.Name;
                    TimelapseExtensions.Add(item);
                    Attributes.Add(attr,item);
                    Console.WriteLine("Before OnInit");
                    item.OnInit();
                     Console.WriteLine("After OnInit");
                }
            }
            Assemblies.Add(asm,dll);
        }
        /// <summary>
        /// Assemblies
        /// </summary>
        /// <value>Dictionary of assembly,string</value>
          public static Dictionary<Assembly,string> Assemblies {get;set;}
          /// <summary>
          /// Stores Attributes for extension
          /// </summary>
          /// <value>returns timelapseExtension based on attribute</value>
        public static Dictionary<ExtensionAttribute,TimelapseExtension> Attributes {get;set;}
        /// <summary>
        /// List of extensions
        /// </summary>
        /// <value>List of extensions</value>
        public static List<TimelapseExtension> TimelapseExtensions {get;set;}
        /// <summary>
        /// Dont call this (this gets called from The UI)
        /// </summary>
       
        public static async Task OnNewFrame(Image<Rgb24> image)
        {
            foreach(var extension in TimelapseExtensions)
            {
                await extension.OnNewFrame(image);
            }
        }
        /// <summary>
        /// Dont call this (this gets called from The UI)
        /// </summary>
        public static void OnLoadProject(TimelapseProject project)
        {
            foreach(var extension in TimelapseExtensions)
            {
                extension.OnLoadProject(project);
            }
        }
        /// <summary>
        /// Dont call this (this gets called from The UI)
        /// </summary>
        public static void OnStopRecording()
        {
            foreach(var extension in TimelapseExtensions)
            {
                extension.OnStopRecording();
            }
        }
        /// <summary>
        /// Dont call this (this gets called from The UI)
        /// </summary>
        public static void OnStartRecording()
        {
            foreach(var extension in TimelapseExtensions)
            {
                extension.OnStartRecording();
            }
        }
        /// <summary>
        /// Dont call this (this gets called from The UI)
        /// </summary>
        public static void OnEnableOneX()
        {
            foreach(var extension in TimelapseExtensions)
            {
                extension._onex=true;
                extension.OnEnableOneX();
            }
        }
        /// <summary>
        /// Dont call this (this gets called from The UI)
        /// </summary>
         public static void OnDisableOneX()
        {
            foreach(var extension in TimelapseExtensions)
            {
                extension._onex=false;
                extension.OnEnableOneX();
            }
        }
    }
}
